        page    64,127
        title   SHV - Personal REXX Shared Variable Interface Example
        comment @

This is SHV.ASM, an example of the Personal REXX Shared Variable Interface.

Function:

        Display the value of a REXX variable

Entry Conditions:

        This program should be run from a REXX program.
        The name of a REXX variable should be passed on the command line.

Exit Conditions:

        This program exits with the REXX Shared Variable Interface return code.

Include Files:

        REXXCALL.MAC - maps REXX control block
        CALLREXX.ASM - gets address of REXX interrupt handler
        FINDREXX.ASM - gets REXX interrupt number

Sample Usage:

        parse version v /* put version number into variable "v" */
        'shv v'         /* display value of variable "v" */

To recreate SHV.COM (the executable version of this program):

        MASM SHV;
        LINK SHV;
        EXE2BIN SHV SHV.COM

        ERASE SHV.OBJ
        ERASE SHV.EXE

This program was tested with Personal REXX 1.60, DOS 3.3, and MASM 4.0.

@
        page
        include rexxcall.mac    ; map REXX control blocks
        page
code    segment
        assume  cs:code,ds:code
        org     80H
parml   db      ?               ; parameter length
        db      ?               ; separator
parms   db      ?               ; user's parms
        org     100h            ; org past PSP
begin:
        jmp     init            ; branch around data area
sb              shvblock <>     ; shared variable request block
sb_length       equ     $-sb

buffer  db      250 dup (?)     ; place for REXX to put value
buffer_length   equ     $-buffer

no_rexx         db      'Error: REXX not loaded.$'

crlf    db      13,10           ; carriage return/line feed
        page
        include callrexx.asm    ; include getint and errint routines
        page
init:
;       get address of REXX interrupt handler
;       -------------------------------------
        call    getint          ; get addr of REXX interrupt handler
        jz      gotint          ; if no error, continue
        jmp     errint          ; else print error message and exit
gotint:                         ; continue ...

;       insure REXX is loaded - abort if not
;       ------------------------------------
        mov     ah,rexx_version         ; function = get Personal REXX version
        mov     al,0                    ; default major version = 0
        pushf                           ; save flags for REXX's iret
        call    callrexx                ; get version number in ax
        or      al,al                   ; did major version change?
        jnz     loaded                  ; yes, then REXX is loaded, so continue
        lea     dx,no_rexx              ; no, get address of error message
        jmp     error                   ; write error message and exit
loaded:

;       Note: during the rest of this routine BX contains segment address
;       -----------------------------------------------------------------
        mov     bx,ds

;       fill in shvblock
;       ----------------

;       zero shvblock
        cld
        mov     al,0                    ; fill character
        lea     di,sb                   ; address of area to zero
        mov     cx,sb_length            ; length of area to zero
        rep     stosb                   ; zero the shvblock

;       set function code
        mov     sb.shvcode,SHVSYFET     ; function = fetch symbolic variable

;       address of variable to be fetched (user's parm)
        lea     ax,parms
        mov     word ptr sb.shvnam_addr+0,ax
        mov     word ptr sb.shvnam_addr+2,bx

;       length of variable to be fetched
        mov     al,parml                ; get length of parms
        dec     al                      ; minus 1 for separator
        mov     byte ptr sb.shvnaml,al  ; save length in shvblock

;       address and length of buffer (where to put value of variable)
        lea     ax,buffer
        mov     word ptr sb.shvval_addr+0,ax
        mov     word ptr sb.shvval_addr+2,bx
        mov     byte ptr sb.shvbufl,buffer_length

;       set up registers and call REXX
;       ------------------------------
        mov     ah,REXX_SHV             ; shared variable interface function
        mov     bx,ds                   ; get segment address of shvblock
        lea     dx,sb                   ; get offset to shvblock
        pushf                           ; save flags for REXX's iret
go:     call    callrexx                ; call REXX interrupt handler
        page
;       if successful, display value of variable
;       ----------------------------------------

;       check for success
        mov     al,sb.shvret            ; get return code
        cmp     al,1                    ; success?
        ja      return                  ; no, exit with REXX's return code

;       display value of variable
        mov     ah,40h                  ; dos function = write to file or device
        mov     bx,1                    ; file handle = stdout
        lea     dx,buffer               ; offset of string to write
        mov     cx,word ptr sb.shvvall  ; get length of string
        int     21h                     ; call dos to display string
        mov     ah,40h                  ; dos function = write to file or device
        mov     bx,1                    ; file handle = stdout
        lea     dx,crlf                 ; offset to cr/lf pair
        mov     cx,2                    ; get length of string
        int     21h                     ; call dos to write cr/lf

;       return to DOS
;       -------------
return:
        mov     al,sb.shvret            ; return code = REXX return code
        mov     ah,4Ch                  ; DOS func = exit
        int     21H                     ; Return to DOS

;       Error exit routine - DX has offset of error message.
;       ----------------------------------------------------

error   proc    near
        mov     ah,9    ; dos function = print string
        int     21h     ; call dos to print error message
        ret             ; return to dos
error   endp

code    ends
        end     begin
