/*

This is SHV.C, an example of the Personal REXX Shared Variable Interface.
Coded for Microsoft C version 5.1 small, medium, or large models.

Function:

    Test Personal REXX Shared Variable Interface.

Entry Conditions:

    This program should be run from a REXX program.
    Any parameters are ignored.

Exit Conditions:

    This program exits with a return code of 255.

Include Files:

    REXXCALL.H  - maps REXX control block

Calls:

    FINDINT.ASM - gets address of REXX interrupt handler

Sample Usage:

    SHV should be run from a REXX program. SHV prompts for the
    function (shvcode) and variable name (shvnam).  If a set
    variable function is specified, it also prompts for the value
    (shvval).  After calling REXX, SHV displays the return code
    (shvret).  The value of the variable (shvval) is displayed for
    fetch functions.  This process is repeated until the user enters
    a null line or presses ctrl-break.

Notes:

    To recreate executable version of this program use the command

        MAKE APIMSC                     (small model)
        MAKE MODEL=-AM APIMSC           (medium model)
        MAKE MODEL=-AL APIMSC           (large model)

    MYAPPL.C uses C preprocessor statements to generate
    the correct code for any of the three models.

    FINDINT.ASM also needs no changes for the different models.

    This program was tested with Personal REXX 2.00, DOS 3.3, and MSC 5.1.

*/

#include <stdio.h>
#include <dos.h>
#include <malloc.h>

#include "rexxcall.h"

#define MAX_STR     250     /* max size of variable and value */
#define MAX_CODE    1       /* max size of shvcode */

#ifdef M_I86SM  /* small model */
#define FINDINT findints
#endif
#ifdef M_I86MM  /* medium model */
#define FINDINT findintm
#endif
#ifdef M_I86LM  /* large model */
#define FINDINT findintl
#endif

main()
{
    unsigned data_segment;
    int i,rexx_interrupt,interrupt_status;
    /* note, +3s below are for \n, \0, and an extra byte */
    char shvnam[MAX_STR+3],shvval[MAX_STR+3],shvcode[MAX_CODE+3],*cptr;

    struct shvblock sb,*sbp = &sb;
    union REGS inregs, outregs;
    struct SREGS segregs;

    printf("Personal Rexx Shared Variable Interface Demo \n");

    rexx_interrupt = FINDINT(&interrupt_status);
    if (interrupt_status != 0) error("RXINTMGR not installed.\n");

    inregs.h.ah = REXX_VERSION;
    inregs.h.al = 0;
    int86(rexx_interrupt, &inregs, &outregs);
    if (outregs.h.al == 0) error("REXX not loaded\n");

    segread(&segregs);
    data_segment = segregs.ds;

    printf("Enter a null line at any time to exit\n");

    for (;;) {

    memset(&sb, 0, sizeof(struct shvblock));

    get("SHVCODE: ",shvcode,sizeof(shvcode));
    memcpy(&sb.shvcode,shvcode,1);

    get("SHVNAM:  ",shvnam,sizeof(shvnam));
    sb.shvnam_addr = (FARP) shvnam;
    sb.shvnaml = strlen(shvnam)-1;

    if (sb.shvcode == 's' || sb.shvcode == 'S') {   /* shvvall or shvbufl */
        get("SHVVAL:  ",shvval,sizeof(shvval));
        sb.shvvall = strlen(shvval)-1;
    }
    else {
       sb.shvbufl = sizeof(shvnam);
    }

    sb.shvval_addr = (FARP) shvval;     /* get shvval */

    inregs.h.ah = REXX_SHV;
#ifdef M_I86LM    /* predefined by microsoft c if large model */
    inregs.x.bx = FP_SEG(sbp);
    inregs.x.dx = FP_OFF(sbp);
#else       /* medium or small model */
    inregs.x.bx = data_segment;
    inregs.x.dx = (unsigned) sbp;
#endif
    int86x(rexx_interrupt, &inregs, &outregs, &segregs);
    if (outregs.x.ax == -2) error("out of memory");

    printf("SHVRET:  %x (hex)\n",(unsigned char) sb.shvret);

    if (toupper(sb.shvcode)=='F'||sb.shvcode=='P'){ /* display shvval */
        printf("SHVVALL: %d\n",sb.shvvall);
        printf("SHVVAL:  ");
        for(i=0;i<sb.shvvall;i++)
           putc(shvval[i],stdout);
        putc('\n',stdout);
    }
    putc('\n',stdout);
    }
}
/* display prompt, then get response (max length is in size) */
get(prompt,response,size)
char *prompt,*response;
unsigned int size;
{
    char *cptr;

    printf(prompt);
    cptr = fgets(response,size,stdin);
    if (!cptr) error("eof or error\n");
    if (strlen(response) > size-2) error("too long\n");
    if (strcmp(response,"\n")==0) error("null string entered\n");
    return;
}

/* display error message and exit (similar to ci 86 abort() function) */
error(s)
char *s;
{
    printf("ABORT:- %s\n",s);
    exit(255);
}
